# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Annotated
from typing import TypeAlias

import fastapi

from oauthx.lib.protocols import BaseSettings


__all__: list[str] = [
    'CurrentSettings',
    'CurrentSettingsDependency',
]


def get(request: fastapi.Request) -> BaseSettings:
    return request.config # type: ignore



CurrentSettingsDependency: BaseSettings = fastapi.Depends(get)
CurrentSettings: TypeAlias = Annotated[BaseSettings, CurrentSettingsDependency]